package csspolice;

/**
 * <p>Title: </p>
 * <p>Description: Main program invoking css->xml->xsl->(xhtml|plain)-conversion</p>
 * <p>Copyright: Copyright (c) 2005, 2006 Jukka Mntyl</p>
 * <p>Company: Jyvskyln yliopisto</p>
 * <p>Changes:
 * 1.02 license text
 * 1.01 removed xhtml-output, url is now required
 * 1.0 options for xml-base
 * </p>
 * <p>Requires:
 * - CSS Parser http://cssparser.sourceforge.net/
 * - JSAP       http://www.martiansoftware.com/jsap/
 * - SAC        http://www.w3.org/Style/CSS/SAC/
 * </p>
 * @author Jukka Mntyl
 */

/*
 License

 This file is part of CSStoXML.

     CSStoXML is free software; you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
     the Free Software Foundation; either version 2.1 of the License, or
     (at your option) any later version.

     CSStoXML is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU Lesser General Public License for more details.

     You should have received a copy of the GNU Lesser General Public License
     along with CSStoXML; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

import com.steadystate.css.parser.*;
import com.martiansoftware.jsap.*;
import java.net.*;
import java.io.*;

import org.w3c.css.sac.CSSException;
import javax.xml.transform.*;

public class CSSPolice {

  private static void printUsage(JSAP jsap) {
    System.err.println("Error in parsing command-line parameters.\nUSAGE: java -jar csstoxml.jar");
    System.err.println("       " + jsap.getUsage());
    System.err.println();
    System.err.println(jsap.getHelp());
  }

  public static void main(String[] args) {

    // options
    String url = "";
    SACMediaListImpl medialist = null;
    String transform = "";
    boolean form = false;
    String result = "";
    StringBuffer virheet = new StringBuffer("");

    // parse options

    JSAP jsap = new JSAP();
    FlaggedOption opt1 = new FlaggedOption("xslstylesheet");
    opt1.setDefault("");
    opt1.setStringParser(new com.martiansoftware.jsap.stringparsers.StringStringParser());
    opt1.setRequired(false);
    opt1.setLongFlag("xslsheet");
    opt1.setShortFlag('x');
    opt1.setHelp("XSL-stylesheet for transformation");

    FlaggedOption opt4 = new FlaggedOption("media");
    opt4.setStringParser(new com.martiansoftware.jsap.stringparsers.StringStringParser());
    opt4.setDefault("");
    opt4.setRequired(false);
    opt4.setLongFlag("media");
    opt4.setShortFlag('m');
    opt4.setHelp("Comma-separated medialist for CSS-stylesheet. Must not contain spaces.");


    UnflaggedOption opt3 = new UnflaggedOption("url").setStringParser(new com.martiansoftware.jsap.stringparsers.StringStringParser()).setDefault("");
    opt3.setRequired(true);
    opt3.setHelp("Valid URL containing CSS2-stylesheet.");

    try {
      jsap.registerParameter(opt3);
      jsap.registerParameter(opt1);
      jsap.registerParameter(opt4);
    } catch (JSAPException j) {
      printUsage(jsap);
      System.exit(1);
    }

    JSAPResult config = jsap.parse(args);
    if (!config.success()) {

      printUsage(jsap);
    }

    transform = config.getString("xslstylesheet");

    String query = System.getProperty("cgi.query_string");

    // if there is not parameters in url then use command-line parameters
    if (query == null) {
      url = config.getString("url");
      String mediaargs = config.getString("media");
      if (!mediaargs.equals("")) {
        String[] medias = mediaargs.split(",");
        medialist = new SACMediaListImpl();
        for (int i = 0; i < medias.length; i++)
          medialist.add(medias[i].trim());
      }

    } else {
      try {

        String querydec = URLDecoder.decode(query, "UTF-8");
        String[] pairs = querydec.split("&");

        for (int i = 0; i < pairs.length; i++) {
          String[] keyval = pairs[i].split("=");
          if (keyval.length > 1) {
            if (keyval[0].equals("url") && keyval[1].length() > 0)
              url = keyval[1];
            if (keyval[0].equals("media") && keyval[1].length() > 0) {
              String[] medias = keyval[1].split(",");
              medialist = new SACMediaListImpl();
              for (int j = 0; i < medias.length; i++)
                medialist.add(medias[j].trim());
            }
          }
        }
      }
      catch (UnsupportedEncodingException e) {
        System.err.println("Problem in parsing parameters.");
        System.exit(1);
      }
    }

    // css to xml-conversion
    if (url.length() > 0) {
      try {
        CSSconvertXML converter = new CSSconvertXML(url, medialist, virheet);
        // transform through xsl-stylesheet
        if (transform.length() > 0) {
          CSSPoliceTransformer transformer = new CSSPoliceTransformer(converter.
              getReader(),transform);
          result = transformer.getResults();
        }
        else {
          result = converter.getString();
        }
      } catch (CSSException ex) {
        virheet.append("CSS-parser: Fatal parse error. Validate your CSS-stylesheet.\n");
      } catch (MalformedURLException ex) {
        virheet.append("CSS-parser: The form of URL-address in incorrect.\n");
      } catch (java.io.IOException ex) {
        virheet.append("CSS-parser: I/O-error. Check you URL-address.\n");
      } catch (TransformerConfigurationException e) {
        virheet.append("XSL-processor: Error in configuration.");
      } catch (TransformerException e) {
        virheet.append("XSL-processor: Error in transformation.");
      } catch (java.lang.Error e) {
        virheet.append("CSS-parser: Parse error. Your CSS-stylesheet does not follow CSS2-recommendation.");
      }

    }

   if (url.length() > 0)
        System.out.println(result);
      else {
        System.err.println("Not enough parameters.");
        printUsage(jsap);
      }
      if (virheet.length() > 0)
        System.err.println(virheet.toString());
    }


}
