package csspolice;

import org.w3c.css.sac.DocumentHandler;
import org.w3c.css.sac.InputSource;
import org.w3c.css.sac.CSSException;
import org.w3c.css.sac.SACMediaList;
import org.w3c.css.sac.SelectorList;
import org.w3c.css.sac.LexicalUnit;

import com.steadystate.css.parser.LexicalUnitImpl;
import com.steadystate.css.parser.*;


/**
 * <p>Description: Print CSS as XML to stringbuffer</p>
 * <p>Copyright: Copyright (c) 2005, 2006</p>
 * <p>Company: jyu</p>
 * <p>Update date: 9.11.2006</p>
 * <p>Changes:
 * 1.02 DTD now included, removed unnecessary atrule & namespace
 * </p>
 * @author Jukka Mntyl
 * @version 1.02
 * @todo make a parser which is able to create DOM-tree
 */

/*
 License

 This file is part of CSStoXML.

     CSStoXML is free software; you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
     the Free Software Foundation; either version 2.1 of the License, or
     (at your option) any later version.

     CSStoXML is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU Lesser General Public License for more details.

     You should have received a copy of the GNU Lesser General Public License
     along with CSStoXML; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*

types:

// operators
SAC_OPERATOR_COMMA	= 0;
SAC_OPERATOR_PLUS	= 1;
SAC_OPERATOR_MINUS	= 2;
SAC_OPERATOR_MULTIPLY	= 3;
SAC_OPERATOR_SLASH	= 4;
SAC_OPERATOR_MOD	= 5;
SAC_OPERATOR_EXP	= 6;
SAC_OPERATOR_LT		= 7;
SAC_OPERATOR_GT		= 8;
SAC_OPERATOR_LE		= 9;
SAC_OPERATOR_GE		= 10;
SAC_OPERATOR_TILDE	= 11;
SAC_INHERIT		= 12;
// integer
SAC_INTEGER		= 13;
// number
SAC_REAL		= 14;
// lenghts
SAC_EM			= 15;
SAC_EX			= 16;
SAC_PIXEL		= 17;
SAC_INCH		= 18;
SAC_CENTIMETER		= 19;
SAC_MILLIMETER		= 20;
SAC_POINT		= 21;
SAC_PICA		= 22;
SAC_PERCENTAGE		= 23;
// uri
SAC_URI		        = 24;
SAC_COUNTER_FUNCTION	= 25;
SAC_COUNTERS_FUNCTION	= 26;
SAC_RGBCOLOR		= 27;
SAC_DEGREE		= 28;
SAC_GRADIAN		= 29;
SAC_RADIAN		= 30;
SAC_MILLISECOND		= 31;
SAC_SECOND		= 32;
SAC_HERTZ		= 33;
SAC_KILOHERTZ		= 34;
SAC_IDENT		= 35;
SAC_STRING_VALUE	= 36;
SAC_ATTR		= 37;
SAC_RECT_FUNCTION	= 38;
SAC_UNICODERANGE	= 39;
SAC_SUB_EXPRESSION	= 40;
SAC_FUNCTION		= 41;
SAC_DIMENSION		= 42;

selectors:

SAC_CONDITIONAL_SELECTOR = 0;
SAC_ANY_NODE_SELECTOR = 1;
SAC_ROOT_NODE_SELECTOR = 2;
SAC_NEGATIVE_SELECTOR = 3;
SAC_ELEMENT_NODE_SELECTOR = 4;
SAC_TEXT_NODE_SELECTOR = 5;
SAC_CDATA_SECTION_NODE_SELECTOR = 6;
SAC_PROCESSING_INSTRUCTION_NODE_SELECTOR = 7;
SAC_COMMENT_NODE_SELECTOR = 8;
SAC_PSEUDO_ELEMENT_SELECTOR = 9;
SAC_DESCENDANT_SELECTOR = 10;
SAC_CHILD_SELECTOR = 11;
SAC_DIRECT_ADJACENT_SELECTOR = 12;


*/


public class PrintXMLHandler implements DocumentHandler {
  public PrintXMLHandler() {
    try {
      jbInit();
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  public static final int sisennysmaara = 2;

  private int spaces = 0;
  private SACMediaList defaultmedia;
  private StringBuffer s;

  private static StringBuffer EntityEncode( String s ) {
    StringBuffer buf = new StringBuffer();
    for ( int i = 0; i < s.length(); i++ ) {
        char c = s.charAt( i );
        switch ( c ) {
          case '&': buf.append("&amp;"); break;
          case '<': buf.append("&lt;"); break;
          case '>': buf.append("&gt;"); break;
          case '\'': buf.append("&apos;"); break;
          case '\"': buf.append("&quot;"); break;
          default: buf.append( c );
        }
    }
    return buf;
  }

  public PrintXMLHandler(StringBuffer s, SACMediaList media) {
    this.s = s;
    if (media == null) {
      SACMediaListImpl newmedia = new SACMediaListImpl();
      newmedia.add("all");
      defaultmedia = newmedia;
    }
    else
      defaultmedia = media;
  }

  private void sisennys() {
      spaces += sisennysmaara;
  }

  private void ulonnus() {
      spaces -= sisennysmaara;
  }

  private void sisenna() {
      for (int i = 0; i < spaces; i++) {
          s.append(" ");
      }
  }


  private void pr(String text) {
    sisenna();
    s.append(text);
  }

  private void prln(String text) {
    sisenna();
    s.append(text);
    s.append("\n");
  }

  private void cpr(String text) {
    s.append(text);
  }

  private void cprln(String text) {
    s.append(text);
    s.append("\n");
  }

  public void startDocument(InputSource source) throws CSSException {

      prln("<?xml version=\"1.0\" encoding=\"ISO-8859-1\" ?>");
      prln("<!DOCTYPE css [");
      prln("  <!ELEMENT css (media)* >");
      prln("  <!ELEMENT media (import|page|font-face|media|selector)*>");
      prln("  <!ATTLIST media type CDATA \"all\">");
      prln("  <!ELEMENT import (url+)>");
      prln("  <!ATTLIST import media CDATA #IMPLIED>");
      prln("  <!ELEMENT url (#PCDATA)>");
      prln("  <!ELEMENT page (property*)>");
      prln("  <!ATTLIST page name CDATA #IMPLIED>");
      prln("  <!ATTLIST page selector CDATA #IMPLIED>");
      prln("  <!ELEMENT font-face (property*)>");
      prln("  <!ELEMENT selector (simple+,property+)>");
      prln("  <!ELEMENT simple (#PCDATA)>");
      prln("  <!ATTLIST simple type CDATA \"0\">");
      prln("  <!ELEMENT property (name,value+)>");
      prln("  <!ATTLIST property important CDATA #IMPLIED>");
      prln("  <!ELEMENT name (#PCDATA)>");
      prln("  <!ELEMENT value (operator|inherit|integer|number|unit|url|text|nest)*>");
      prln("  <!ATTLIST value type CDATA #REQUIRED>");
      prln("  <!ELEMENT operator (#PCDATA)>");
      prln("  <!ELEMENT inherit EMPTY>");
      prln("  <!ELEMENT integer (#PCDATA)>");
      prln("  <!ELEMENT number (#PCDATA)>");
      prln("  <!ELEMENT unit (#PCDATA)>");
      prln("  <!ELEMENT text (#PCDATA)>");
      prln("  <!ELEMENT nest (name,params)>");
      prln("  <!ELEMENT params (operator|integer|number|unit|url|text|param)*>");
      prln("  <!ELEMENT param (#PCDATA)>");
      prln("]>");
      prln("<css>");
      sisennys();
      if (defaultmedia != null)
        startMedia(defaultmedia);
  }

  public void endDocument(InputSource source) throws CSSException {
      if (defaultmedia != null)
        endMedia(defaultmedia);
      ulonnus();
      prln("</css>");
  }

  public void comment(String text) throws CSSException {
  }

  public void ignorableAtRule(String atRule) throws CSSException {
  }

  public void namespaceDeclaration(String prefix, String uri) throws CSSException {
  }

  public void importStyle(String uri, SACMediaList media,
                      String defaultNamespaceURI)
          throws CSSException {
      pr("<import");
      if (media.getLength() > 0)
        cpr(" media=\"" + media.toString() + "\"");
      cprln(">");
      sisennys();
        prln("<url>" + EntityEncode(uri) + "</url>");
      ulonnus();
      prln("</import>");
  }

  public void startMedia(SACMediaList media) throws CSSException {
    pr("<media");
    if (media.getLength() > 0)
      cpr(" type=\"" + EntityEncode(media.toString()) + "\"");
    else
      cpr(" type=\"all\"");
    cprln(">");
    sisennys();
  }

  public void endMedia(SACMediaList media) throws CSSException {
      ulonnus();
      prln("</media>");
  }

  public void startPage(String name, String pseudo_page) throws CSSException {
      pr("<page");
      if (name != null) {
        cpr(" name=\"" + EntityEncode(name) + "\"");
      }
      if (pseudo_page != null) {
        cpr(" selector=\"" + EntityEncode(pseudo_page) + "\"");
      }
      cprln(">");
      sisennys();
  }

  public void endPage(String name, String pseudo_page) throws CSSException {
      ulonnus();
      prln("</page>");
  }

  public void startFontFace() throws CSSException {
      prln("<font-face>");
      sisennys();
  }

  public void endFontFace() throws CSSException {
      ulonnus();
      prln("</font-face>");
  }

  public void startSelector(SelectorList selectors) throws CSSException {
      prln("<selector>");
      sisennys();
      for(int i=0;i<selectors.getLength();i++)
        prln("<simple type=\"" + selectors.item(i).getSelectorType() + "\">" + EntityEncode(selectors.item(i).toString()) + "</simple>");
  }

  public void endSelector(SelectorList selectors) throws CSSException {
      ulonnus();
      prln("</selector>");
  }

  public void property(String name, LexicalUnit value, boolean important) throws CSSException {
      pr("<property");
      if (important)
        cpr(" important=\"true\"");
      cprln(">");
      sisennys();
      prln("<name>" + EntityEncode(name) + "</name>");

      // Let's get through linked list of attribute values
      LexicalUnit val = value;
      int valtype;
      while (val != null) {
          // no commas
          valtype = val.getLexicalUnitType();
          if (valtype != LexicalUnit.SAC_OPERATOR_COMMA) {
            prln("<value type=\"" + valtype + "\">");
            sisennys();
            printElement(val);
            ulonnus();
            prln("</value>");
          }
          val = val.getNextLexicalUnit();
      }
      ulonnus();
      prln("</property>");
  }

  private void printElement(LexicalUnit val) {
      LexicalUnit l = null;
      switch (val.getLexicalUnitType()) {
         case LexicalUnit.SAC_OPERATOR_COMMA:
           break;
         case LexicalUnit.SAC_OPERATOR_PLUS:
           prln("<operator>+</operator>");
           break;
         case LexicalUnit.SAC_OPERATOR_MINUS:
           prln("<operator>-</operator>");
           break;
         case LexicalUnit.SAC_OPERATOR_MULTIPLY:
           prln("<operator>*</operator>");
           break;
         case LexicalUnit.SAC_OPERATOR_SLASH:
           prln("<operator>/</operator>");
           break;
         case LexicalUnit.SAC_OPERATOR_MOD:
           prln("<operator>mod</operator>");
           break;
         case LexicalUnit.SAC_OPERATOR_EXP:
           prln("<operator>^</operator>");
           break;
         case LexicalUnit.SAC_OPERATOR_LT:
           prln("<operator>&lt;</operator>");
           break;
         case LexicalUnit.SAC_OPERATOR_GT:
           prln("<operator>&gt;</operator>");
           break;
         case LexicalUnit.SAC_OPERATOR_LE:
           prln("<operator>&lt;=</operator>");
           break;
         case LexicalUnit.SAC_OPERATOR_GE:
           prln("<operator>&gt;=</operator>");
           break;
         case LexicalUnit.SAC_OPERATOR_TILDE:
           pr("<operator>~</operator>");
           break;
         case LexicalUnit.SAC_INHERIT:
           prln("<inherit />");
           break;
         case LexicalUnit.SAC_INTEGER:
           prln("<integer>" + String.valueOf(val.getIntegerValue()) + "</integer>");
           break;
         case LexicalUnit.SAC_REAL:
           prln("<number>" + val.getFloatValue() + "</number>");
           break;
         case LexicalUnit.SAC_EM:
         case LexicalUnit.SAC_EX:
         case LexicalUnit.SAC_PIXEL:
         case LexicalUnit.SAC_INCH:
         case LexicalUnit.SAC_CENTIMETER:
         case LexicalUnit.SAC_MILLIMETER:
         case LexicalUnit.SAC_POINT:
         case LexicalUnit.SAC_PICA:
         case LexicalUnit.SAC_PERCENTAGE:
         case LexicalUnit.SAC_DEGREE:
         case LexicalUnit.SAC_GRADIAN:
         case LexicalUnit.SAC_RADIAN:
         case LexicalUnit.SAC_MILLISECOND:
         case LexicalUnit.SAC_SECOND:
         case LexicalUnit.SAC_HERTZ:
         case LexicalUnit.SAC_KILOHERTZ:
         case LexicalUnit.SAC_DIMENSION:
           prln("<number>" + val.getFloatValue() + "</number>");
           prln("<unit>" + val.getDimensionUnitText() + "</unit>");
           break;
         case LexicalUnit.SAC_URI:
           prln("<url>" + EntityEncode(val.getStringValue()) + "</url>");
           break;
         case LexicalUnit.SAC_RGBCOLOR:
         case LexicalUnit.SAC_COUNTER_FUNCTION:
         case LexicalUnit.SAC_COUNTERS_FUNCTION:
         case LexicalUnit.SAC_RECT_FUNCTION:
         case LexicalUnit.SAC_FUNCTION:
           prln("<nest>");
           sisennys();
           prln("<name>" + EntityEncode(val.getFunctionName()) + "</name>");
           prln("<params>");
           sisennys();
           l = val.getParameters();
           while (l != null) {
              printElement(l);
              l = l.getNextLexicalUnit();
           }
           ulonnus();
           prln("</params>");
           ulonnus();
           prln("</nest>");
           break;
         case LexicalUnit.SAC_IDENT:
         case LexicalUnit.SAC_STRING_VALUE:
         case LexicalUnit.SAC_ATTR:
         case LexicalUnit.SAC_UNICODERANGE:
           prln("<text>" + EntityEncode(val.getStringValue()) + "</text>");
           break;
         case LexicalUnit.SAC_SUB_EXPRESSION:
           prln("<nest>");
           sisennys();
           prln("<params>");
           sisennys();
           l = val.getSubValues();
           while (l != null) {
              prln("<param>" + EntityEncode(l.getStringValue()) + "</param>");
              l = l.getNextLexicalUnit();
           }
           ulonnus();
           prln("</params>");
           ulonnus();
           prln("</nest>");
           break;
         default:
           break;
       }
    }

  private void jbInit() throws Exception {
  }

}
